% MTODO 3
% TEST 3: TEST BDS
% VERSIN 1
%
% Las referencias utilizadas han sido, principalmente:
%
%	W.A. Brock, W.D. Dechert, J.A. Scheinkman, B. LeBaron
%	A TEST FOR INDEPENDENCE BASED ON THE CORRELATION DIMENSION
%	Econometric Reviews 15 (1996), 197-235
%
%	L. Kanzler
%	VERY FAST AND CORRECTLY SIZED ESTIMATION OF THE BDS STATISTIC
%	Department of Economics of Oxford University (1999)
%
% Para implementar el algoritmo, los pasos a seguir son:
%
% PASO 1: Introducimos la serie temporal con la que vamos a trabajar.
% PASO 2: Calculamos el estadstico de contraste Wmn.
% PASO 2.1: Calculamos Tmn.
% PASO 2.2: Calculamos Vmn.
% PASO 2.3: Calculamos Wmn.
% PASO 3: Realizamos el contraste de hiptesis.

function t3=test_3(serie_temp,m,parametro_Fcaracteristica,nivel_confianza)

% PASO 1
% INTRODUCIMOS LA SERIE TEMPORAL CON LA QUE VAMOS A TRABAJAR

serie=serie_temp; % serie temporal
n=length(serie); % nmero de elementos de la serie
Ntrozos=n-m+1; % nmero de trozos en que se divide la serie

% PASO 2
% CALCULAMOS EL ESTADSTICO DE CONTRASTE Wmn

desviacion_tipica=std(serie(:)); % desviacin tpica de los elementos de la serie
epsilon=parametro_Fcaracteristica*desviacion_tipica; %parmetro de distancia

% PASO 2.1
% CALCULAMOS Tmn

suma_Cmn=0;

suma_C1n=0;

Ncombinaciones=Ntrozos*(Ntrozos-1)/2;

% CALCULAMOS Cmn

for cont1=1:Ntrozos-1
    
    trozo_serie_Cmn=serie(cont1:cont1+m-1);
    
    for cont2=cont1+1:Ntrozos
        
        diferencia_Cmn=trozo_serie_Cmn-serie(cont2:cont2+m-1);
        norma_diferencia_Cmn=norm(diferencia_Cmn,inf);
        
        if(norma_diferencia_Cmn<epsilon)
        
        suma_Cmn=suma_Cmn+1;
        
        end
        
    end
    
end

Cmn=suma_Cmn/Ncombinaciones; % valor de Cmn

% CALCULAMOS C1n

for cont3=m:n-1
    
    elemento_serie_C1n=serie(cont3);
    
    for cont4=cont3+1:n
        
        diferencia_C1n=elemento_serie_C1n-serie(cont4);
        norma_diferencia_C1n=abs(diferencia_C1n);
        
        if(norma_diferencia_C1n<epsilon)
        
        suma_C1n=suma_C1n+1;
        
        end
        
    end
    
end

C1n=suma_C1n/Ncombinaciones; % valor de C1n

Tmn=Cmn-(C1n)^m; % valor de Tmn

% PASO 2.2
% CALCULAMOS Vmn

suma_Cn=0;

suma_Kn=0;

suma_Vmn=0;

% CALCULAMOS Cn

for cont5=1:n-1
    
    elemento_serie_Cn=serie(cont5);
    
    for cont6=cont5+1:n
        
        diferencia_Cn=elemento_serie_Cn-serie(cont6);
        Vabsoluto_diferencia_Cn=abs(diferencia_Cn);
        
        if(Vabsoluto_diferencia_Cn<epsilon)
        
        suma_Cn=suma_Cn+1;
        
        end
                
    end
    
end
 
Cn=suma_Cn/(n*(n-1)/2); % valor de Cn

% CALCULAMOS Kn

for cont7=1:n-2
    
    elemento_serie_Kn_i=serie(cont7);
    
    for cont8=cont7+1:n-1
        
        elemento_serie_Kn_j=serie(cont8);
        
        diferencia_Kn_ij=elemento_serie_Kn_i-elemento_serie_Kn_j;
        Vabsoluto_diferencia_Kn_ij=abs(diferencia_Kn_ij);
        
        if(Vabsoluto_diferencia_Kn_ij<epsilon) 
            
            for cont9=cont8+1:n
           
                diferencia_Kn_jk=elemento_serie_Kn_j-serie(cont9);
                Vabsoluto_diferencia_Kn_jk=abs(diferencia_Kn_jk);
                
                if(Vabsoluto_diferencia_Kn_jk<epsilon)
        
                    suma_Kn=suma_Kn+1;
        
                end
            
            end
        
        end
        
    end
    
end

Kn=suma_Kn/(n*(n-1)*(n-2)/6); % valor de Kn

% CALCULAMOS Vmn

for cont10=1:m-1
    
    incremento_suma_Vmn=(Kn^(m-cont10))*(Cn^(2*cont10));
    suma_Vmn=suma_Vmn+incremento_suma_Vmn;
    
end

Vmn=2*sqrt(Kn^m+2*suma_Vmn+(m-1)^2*Cn^(2*m)-m^2*Kn*Cn^(2*m-2)); % valor de Vmn

% PASO 2.3
% CALCULAMOS Wmn

Wmn=sqrt(n)*Tmn/Vmn; % valor del estadstico de contraste Wmn

% PASO 3
% REALIZAMOS EL CONTRASTE DE HIPTESIS

if (abs(Wmn)<=norminv((1+nivel_confianza/100)/2,0,1))
    
    t3=1;

else
    
    t3=0;
    
end

% Agregamos al resultado de salida los valores de Tmn, Vmn y Wmn

auxiliar=t3;
t3=[Tmn Vmn Wmn auxiliar];
